/*
 * Copyright 2023 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.map;

import java.util.Map;
import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamResult;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key0;
import dk.vajhoej.isam.local.LocalIsamSource;
import dk.vajhoej.record.RecordException;

/**
 * Class IsamMap provide a Map view of an IsamSource.
 *
 * @param <K> key type
 * @param <R> record type
 */
public class IsamMap<K extends Comparable<K>,R> extends MapBase<K,R> {
	private static Logger log = Logger.getLogger(MapBase.class.getName());
	// **** core logic ****
	/**
	 * Class KeyDispatcher operate on selected lookup key.
	 *
	 */
	public class KeyDispatcher {
		protected int keyIndex;
		/**
		 * Create instance of KeyDispatcher.
		 * @param keyIndex key index
		 */
		public KeyDispatcher(int keyIndex) {
			this.keyIndex = keyIndex;
			log.fine("KeyDispatcher for " + keyIndex + " created");
		}
		/**
		 * Select records where lookup key is value.
		 * @param <K2> key type
		 * @param key lookup key
		 * @return IsamSubMap instance
		 */
		public <K2 extends Comparable<K2>> IsamSubMap<K2,R> is(K2 key) {
			return new IsamSubMap<K2,R>(recordClass, src, keyIndex, key, key);
		}
		/**
		 * Select records where lookup key is in interval.
		 * @param <K2> key type
		 * @param startKey start lookup key interval
		 * @param endKey end lookup key interval
		 * @return IsamSubMap instance
		 */
		public <K2 extends Comparable<K2>> IsamSubMap<K2,R> between(K2 startKey, K2 endKey) {
			return new IsamSubMap<K2,R>(recordClass, src, keyIndex, startKey, endKey);
		}
	}
	protected void all(RecordHandler<K,R> h) {
		try {
			log.finer("Iterate over " + recordClass);
			IsamResult<R> rs = src.readStart(recordClass);
			while(rs.read() && h.process(rs.current()));
			rs.close();
		} catch (IsamException e) {
			// nothing
		} catch (RecordException e) {
			// nothing
		}
	}
	// **** construct ****
	/**
	 * Create instance of IsampMap for RMS.
	 * @param <K> key type
	 * @param <R> record type
	 * @param fnm file name
	 * @param recordClass record class (template)
	 * @return IsamMap wrapper
	 * @throws IsamException if problems
	 */
	public static <K extends Comparable<K>,R> IsamMap<K,R> createIsamMapRMS(String fnm, Class<R> recordClass) throws IsamException {
		return new IsamMap<K,R>(fnm, "dk.vajhoej.vms.rms.IndexSequential", recordClass);
	}
	/**
	 * Create instance of IsampMap for BDB.
	 * @param <K> key type
	 * @param <R> record type
	 * @param fnm file name
	 * @param recordClass record class (template)
	 * @return IsamMap wrapper
	 * @throws IsamException if problems
	 */
	public static <K extends Comparable<K>,R> IsamMap<K,R> createIsamMapBDB(String fnm, Class<R> recordClass) throws IsamException {
		return new IsamMap<K,R>(fnm, "dk.vajhoej.bdb.BDB", recordClass);
	}
	/**
	 * Create instance of IsampMap for VBISAM.
	 * @param <K> key type
	 * @param <R> record type
	 * @param fnm file name
	 * @param recordClass record class (template)
	 * @return IsamMap wrapper
	 * @throws IsamException if problems
	 */
	public static <K extends Comparable<K>,R> IsamMap<K,R> createIsamMapVBISAM(String fnm, Class<R> recordClass) throws IsamException {
		return new IsamMap<K,R>(fnm, "dk.vajhoej.vbisam.VBISAM", recordClass);
	}
	/**
	 * Create instance of IsampMap.
	 * @param fnm file name
	 * @param implementation IsamSource implementation class name
	 * @param recordClass record class (template)
	 * @throws IsamException if problems
	 */
	public IsamMap(String fnm, String implementation, Class<R> recordClass) throws IsamException {
		this(new LocalIsamSource(fnm, implementation, false), recordClass);
	}
	/**
	 * Create instance of IsampMap.
	 * @param src IsamSource instance
	 * @param recordClass record class (template)
	 */
	public IsamMap(IsamSource src, Class<R> recordClass) {
		this.src = src;
		this.recordClass = recordClass;
		log.fine("Instance for " + recordClass + " instantiated");
	}
	// map methods
	@Override
	public boolean isEmpty() {
		try {
			IsamResult<R> rs = src.readStart(recordClass);
			boolean empty = !rs.read();
			rs.close();
			return empty;
		} catch (IsamException e) {
			return true;
		} catch (RecordException e) {
			return true;
		}
	}
	@Override
	public boolean containsKey(Object key) {
		return get(key) != null;
	}
	@Override
	public R get(Object key) {
		try {
			@SuppressWarnings("unchecked")
			K ckey = (K)key;
			return src.read(recordClass, new Key0<K>(ckey));
		} catch (IsamException e) {
			log.finer("get threw: " + e);
			return null;
		} catch (RecordException e) {
			log.finer("get threw: " + e);
			return null;
		}
	}
	@Override
	public R put(K key, R value) {
		try {
			if(!key.equals(getKey(value))) {
				return null;
			}
			if(containsKey(key)) {
				src.update(value);
			} else {
				src.create(value);
			}
			return value;
		} catch (IsamException e) {
			log.finer("put threw: " + e);
			return null;
		} catch (RecordException e) {
			log.finer("put threw: " + e);
			return null;
		}
	}
	@Override
	public R remove(Object key) {
		R value = get(key);
		if(value != null) {
			try {
				@SuppressWarnings("unchecked")
				K ckey = (K)key;
				src.delete(recordClass, new Key0<K>(ckey));
				return value;
			} catch (IsamException e) {
				log.finer("remove threw: " + e);
				return null;
			} catch (RecordException e) {
				log.finer("remove threw: " + e);
				return null;
			}
		} else {
			log.finer("remove non-existing key: " + key);
			return null;
		}
	}
	@Override
	public void putAll(Map<? extends K, ? extends R> m) {
		for(K key : m.keySet()) {
			put(key, m.get(key));
		}
	}
	@Override
	public void clear() {
		for(K key : keySet()) {
			remove(key);
		}
	}
	// **** extra methods ****
	/**
	 * Select primary key as lookup key.
	 * @return key dispatcher
	 */
	public KeyDispatcher key() {
		return new KeyDispatcher(0);
	}
	/**
	 * Select lookup key.
	 * @param key key index
	 * @return key dispatcher
	 */
	public KeyDispatcher key(int key) {
		return new KeyDispatcher(key);
	}
}
