/*
 * Copyright 2023 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.map;

import java.util.Map;
import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamResult;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.isam.Key0;
import dk.vajhoej.record.RecordException;

/**
 * Class IsamSubMap provide a Map view for an IsamResult.
 *
 * @param <K> key type
 * @param <R> record type
 */
public class IsamSubMap<K extends Comparable<K>,R> extends MapBase<K,R> {
	private static Logger log = Logger.getLogger(IsamSubMap.class.getName());
	// **** core logic ****
	private int keyIndex;
	private K startKey;
	private K stopKey;
	protected void all(RecordHandler<K,R> h) {
		try {
			log.finer("Iterate over " + recordClass + " from " + startKey + " to " + stopKey);
			IsamResult<R> rs = src.readGE(recordClass, new Key<K>(keyIndex, startKey));
			while(rs.read()) {
				R rec = rs.current();
				if(getKey(rec, keyIndex).compareTo(stopKey) > 0) {
					break;
				}
				if(!h.process(rec)) {
					break;
				}
			}
			rs.close();
		} catch (IsamException e) {
			// nothing
		} catch (RecordException e) {
			// nothing
		}
	}
	// **** construct ****
	protected IsamSubMap(Class<R> recordClass, IsamSource src, int keyIndex,  K startKey, K stopKey) {
		this.recordClass = recordClass;
		this.src = src;
		this.keyIndex = keyIndex;
		this.startKey = startKey;
		this.stopKey = stopKey;
		log.fine("Instance for " + recordClass + " instantiated");
	}
	// **** map methods ****
	@Override
	public boolean isEmpty() {
		try {
			IsamResult<R> rs = src.readGE(recordClass, new Key<K>(keyIndex, startKey));
			boolean empty = !rs.read();
			rs.close();
			return empty;
		} catch (IsamException e) {
			log.finer("isEmpty threw: " + e);
			return true;
		} catch (RecordException e) {
			log.finer("isEmpty threw: " + e);
			return true;
		}
	}
	@Override
	public boolean containsKey(final Object key) {
		RecordHandler<K,R> h = new RecordHandler<K,R>() {
			private boolean found = false;
			@Override
			public boolean process(R rec) {
				try {
					found = key.equals(getKey(rec, keyIndex));
					return !found;
				} catch (IsamException e) {
					return false;
				}
			}
			@Override
			public Object result() {
				return found;
			}
		};
		all(h);
		return (Boolean)h.result();
	}
	@Override
	public R get(Object key) {
		@SuppressWarnings("unchecked")
		K ckey = (K)key;
		if(ckey.compareTo(startKey) >= 0 && ckey.compareTo(stopKey) <= 0) {
			if(keyIndex == 0) {
				try {
					return src.read(recordClass, new Key0<K>(ckey));
				} catch (IsamException e) {
					log.finer("get threw: " + e);
					return null;
				} catch (RecordException e) {
					log.finer("get threw: " + e);
					return null;
				}
			} else {
				throw new RuntimeException("Query can return multiple entries for non-primary key");
			}
		} else {
			log.finer("get returned null because not in interval");
			return null;
		}
	}
	@Override
	public R put(K key, R value) {
		throw new RuntimeException("Cannot modify query result");
	}
	@Override
	public R remove(Object key) {
		throw new RuntimeException("Cannot modify query result");
	}
	@Override
	public void putAll(Map<? extends K, ? extends R> m) {
		throw new RuntimeException("Cannot modify query result");
	}
	@Override
	public void clear() {
		throw new RuntimeException("Cannot modify query result");
	}
}
