/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.remote;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamResult;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.record.RecordException;

/**
 * Class ObjectServer implement IsamProtocolServer using ObjectStream's.
 */
public class ByteServer implements IsamProtocolServer {
    private static final boolean USEBUFFER = true;
    private static Logger log = Logger.getLogger(ByteServer.class.getName());
    private OutputStream os;
    private InputStream is;
    /**
     * Create instance of ObjectServer.
     * @param is input stream
     * @param os output stream
     */
    public ByteServer(InputStream is, OutputStream os) {
        if(USEBUFFER) {
            this.os = new BufferedOutputStream(os);
            this.is = new BufferedInputStream(is);
        } else {
            this.os = os;
            this.is = is;
        }
    }
    public void process(IsamSource isam) throws IsamException, RecordException {
        IsamResult<?> res = null;
        for(;;) {
            try {
                int cmd = is.read();
                if(cmd < 0) {
                    throw new IsamException("EOF of stream");
                }
                log.finer("Received command " + cmd);
                if(cmd == IsamProtocol.READ) {
                    Class<?> clz = ByteHelper.readClass(is);
                    Key<?> key = ByteHelper.readKey(is);
                    os.write(IsamProtocol.OK);
                    log.fine("Read of class " + clz.getName() + " with key " + key);
                    Object o;
                    synchronized(isam)
                    {
                        o = isam.read(clz, key);
                    }
                    ByteHelper.writeObject(os, log, o);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.CREATE) {
                    Object o = ByteHelper.readObject(is, log);
                    log.fine("Create of class " + o.getClass().getName() + " with value " + o);
                    synchronized(isam)
                    {
                        isam.create(o);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.UPDATE) {
                    Object o = ByteHelper.readObject(is, log);
                    log.fine("Update of class " + o.getClass().getName() + " with value " + o);
                    synchronized(isam)
                    {
                        isam.update(o);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.DELETE) {
                    Class<?> clz = ByteHelper.readClass(is);
                    Key<?> key = ByteHelper.readKey(is);
                    log.fine("Delete of class " + clz.getName() + " with key " + key);
                    synchronized(isam)
                    {
                        isam.delete(clz, key);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.CLOSE) {
                    log.fine("Close");
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                    break;
                } else if(cmd == IsamProtocol.FIND_ALL) {
                    Class<?> t = ByteHelper.readClass(is);
                    log.fine("FindGE of class " + t.getName());
                    synchronized(isam) {
                        res = isam.readStart(t);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.FIND_GE) {
                    Class<?> t = ByteHelper.readClass(is);
                    Key<?> k = ByteHelper.readKey(is);
                    log.fine("FindGE of class " + t.getName() + " with key " + k);
                    synchronized(isam) {
                        res = isam.readGE(t, k);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.FIND_GT) {
                    Class<?> t = ByteHelper.readClass(is);
                    Key<?> k = ByteHelper.readKey(is);
                    log.fine("FindGT of class " + t.getName() + " with key " + k);
                    synchronized(isam) {
                        res = isam.readGT(t, k);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.FIND_LE) {
                    Class<?> t = ByteHelper.readClass(is);
                    Key<?> k = ByteHelper.readKey(is);
                    log.fine("FindLE of class " + t.getName() + " with key " + k);
                    synchronized(isam) {
                        res = isam.readLE(t, k);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.FIND_LT) {
                    Class<?> t = ByteHelper.readClass(is);
                    Key<?> k = ByteHelper.readKey(is);
                    log.fine("FindLT of class " + t.getName() + " with key " + k);
                    synchronized(isam) {
                        res = isam.readLT(t, k);
                    }
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.RESULT_READ) {
                    log.fine("Result read");
                    os.write(IsamProtocol.OK);
                    os.write(res.read() ? 1 : 0);
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.RESULT_CURRENT) {
                    log.fine("Result current");
                    os.write(IsamProtocol.OK);
                    ByteHelper.writeObject(os, log, res.current());
                    os.flush();
                    log.fine("OK");
                } else if(cmd == IsamProtocol.RESULT_CLOSE) {
                    log.fine("Result close");
                    res.close();
                    res = null;
                    os.write(IsamProtocol.OK);
                    os.flush();
                    log.fine("OK");
                } else {
                    System.err.println("Unexpected command: " + cmd);
                }
            } catch (Exception e) {
                try {
                    os.write(IsamProtocol.ERROR);
                    ByteHelper.writeException(os, new IsamException("Remote exception", e));
                    os.flush();
                    log.fine("Not OK");
                } catch (IOException e2) {
                    // not much we can do
                }
            }
        }
        try {
            os.close();
            is.close();
        } catch (IOException e) {
            throw new IsamException("Remote exception", e);
        }
    }
}
