/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.remote;

import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamResult;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.record.RecordException;

/**
 * Class EmulatedIsamSource is a remote ISAM source.
 */
public class RemoteIsamSource implements IsamSource {
    private static Logger log = Logger.getLogger(RemoteIsamSource.class.getName());
    private static final int DEFAULT_VERSION = IsamProtocol.O_V1;
    private static int version = DEFAULT_VERSION;
	private Socket cli;
	private IsamProtocolClient del;
	/**
	 * Set protocol version to use.
	 * @param version protocol version
	 */
	public static void setVersion(int version) {
	    RemoteIsamSource.version = version;
	    log.finer("Default wire protocol version changed to " + version);
	}
	/**
	 * Create instance of RemoteIsamSource.
	 * @param hostname server hostname
	 * @param port server port number
	 * @throws IsamException if problem with ISAM source 
	 */
	public RemoteIsamSource(String hostname, int port) throws IsamException {
		try {
			cli = new Socket(hostname, port);
			log.fine("Connected to " + hostname + " port "  + port);
			switch(version) {
                case IsamProtocol.O_V1:
                    del = new ObjectClient(cli.getInputStream(), cli.getOutputStream());
                    log.finer("Using object streams for wire protocol");
                    break;
                case IsamProtocol.B_V1:
                    del = new ByteClient(cli.getInputStream(), cli.getOutputStream());
                    log.finer("Using byte streams for wire protocol");
                    break;
			    default:
			        throw new IsamException("Unsupported protocol version");
			}
            del.handshake();
		} catch (UnknownHostException e) {
		    throw new IsamException("Remote exception", e);
		} catch (IOException e) {
		    throw new IsamException("Remote exception", e);
		}
	}
	public <T, TK extends Comparable<TK>> T read(Class<T> t, Key<TK> k) throws IsamException, RecordException {
	    return del.read(t, k);
	}
	public <T> void create(T o) throws IsamException, RecordException {
	    del.create(o);
	}
    public <T> void update(T o) throws IsamException, RecordException {
        del.update(o);
    }
    public <T, TK extends Comparable<TK>> void delete(Class<T> t, Key<TK> k) throws IsamException, RecordException {
        del.delete(t, k);
    }
	public void close() throws IsamException {
        try {
            del.close();
            cli.close();
        } catch (IOException e) {
            throw new IsamException("Remote exception", e);
        }
	}
	public <T, TK extends Comparable<TK>> IsamResult<T> readStart(Class<T> t) throws IsamException, RecordException {
		return readXX(IsamProtocol.FIND_ALL, t);
	}
	public <T, TK extends Comparable<TK>> IsamResult<T> readGE(Class<T> t, Key<TK> k) throws IsamException, RecordException {
		return readXX(IsamProtocol.FIND_GE, t, k);
	}
	public <T, TK extends Comparable<TK>> IsamResult<T> readGT(Class<T> t, Key<TK> k) throws IsamException, RecordException {
		return readXX(IsamProtocol.FIND_GT, t, k);
	}
	public <T, TK extends Comparable<TK>> IsamResult<T> readLE(Class<T> t, Key<TK> k) throws IsamException, RecordException {
		return readXX(IsamProtocol.FIND_LE, t, k);
	}
	public <T, TK extends Comparable<TK>> IsamResult<T> readLT(Class<T> t, Key<TK> k) throws IsamException, RecordException {
		return readXX(IsamProtocol.FIND_LT, t, k);
	}
	private <T, TK extends Comparable<TK>> IsamResult<T> readXX(int cmd, Class<T> t) throws IsamException, RecordException {
	    return del.readXX(cmd, t);
	}
	private <T, TK extends Comparable<TK>> IsamResult<T> readXX(int cmd, Class<T> t, Key<TK> k) throws IsamException, RecordException {
	    return del.readXX(cmd, t, k);
	}
}
