/*
 * Copyright 2023 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.test;

import static org.junit.Assert.*;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.emulated.EmulatedIsamSource;
import dk.vajhoej.isam.map.PyIsamMap;

public class TestPyIsamMap {
	private final static String DRIVER = "com.mysql.cj.jdbc.Driver";
	private final static String CONURL = "jdbc:mysql://localhost/Test";
	private final static String UN = "root";
	private final static String PW = "hemmeligt";
	@Before
	public void setup() {
		try {
			Class.forName(DRIVER);
			Connection con = DriverManager.getConnection(CONURL, UN, PW);
			Statement stmt = con.createStatement();
			stmt.executeUpdate("CREATE TABLE data (iv INTEGER NOT NULL, recblob BLOB, PRIMARY KEY(iv))");
			stmt.close();
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	@After
	public void teardown() {
		try {
			Connection con = DriverManager.getConnection(CONURL, UN, PW);
			Statement stmt = con.createStatement();
			stmt.executeUpdate("DROP TABLE data");
			stmt.close();
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
    @Test
    public void testCreateRead() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			m.put(o.getIv(), o);
			Data o2 = m.get(258);
			assertEquals("iv", 258, o2.getIv());
			assertEquals("xv", 123.456, o2.getXv(), 0.0005);
			assertEquals("sv", "ABC     ", o2.getSv());
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testCreateReadMulti() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
            for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				m.put(o.getIv(), o);
            }
            for(int i = 0; i < 10; i++) {
				Data o2 = m.get(258 + i);
				assertEquals("iv", 258 + i, o2.getIv());
				assertEquals("xv", 123.456, o2.getXv(), 0.0005);
				assertEquals("sv", "ABC     ", o2.getSv());
            }
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testReadKeys() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			assertEquals("Before size", 0, m.size());
            for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				m.put(o.getIv(), o);
            }
			assertEquals("After size", 10, m.size());
			int i = 0;
			for(int k : m.keySet()) {
				Data o2 = m.get(k);
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("Start records", 10, i);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testPyReadKeys() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            PyIsamMap<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			assertEquals("Before size", 0, m.size());
            for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				m.put(o);
            }
			assertEquals("After size", 10, m.size());
			int i = 0;
			for(int k : m.keys()) {
				Data o2 = m.get(k);
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("Start records", 10, i);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testReadValues() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			assertEquals("Before size", 0, m.size());
            for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				m.put(o.getIv(), o);
            }
			assertEquals("After size", 10, m.size());
			int i = 0;
			for(Data o2 : m.values()) {
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("Start records", 10, i);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testReadEntries() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			assertEquals("Before size", 0, m.size());
            for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				m.put(o.getIv(), o);
            }
			assertEquals("After size", 10, m.size());
			int i = 0;
			for(Entry<Integer,Data> e : m.entrySet()) {
				Data o2 = e.getValue();
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("Start records", 10, i);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testPyReadEntries() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            PyIsamMap<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			assertEquals("Before size", 0, m.size());
            for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				m.put(o);
            }
			assertEquals("After size", 10, m.size());
			int i = 0;
			for(List<Object> e : m.items()) {
				Data o2 = (Data)e.get(1);
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("Start records", 10, i);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testDelete() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			m.put(o.getIv(), o);
			Data o2 = m.get(258);
			assertNotNull("o2", o2);
			m.remove(258);
			o2 = m.get(258);
			assertNull("o2", o2);
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
    @Test
    public void testUpdated() {
		try {
            IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
            Map<Integer,Data> m = new PyIsamMap<Integer,Data>(isam, Data.class);
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			m.put(o.getIv(), o);
			Data o2 = m.get(258);
			assertEquals("iv", 258, o2.getIv());
			assertEquals("xv", 123.456, o2.getXv(), 0.0005);
			assertEquals("sv", "ABC     ", o2.getSv());
			o2.setXv(123.789);
			o2.setSv("ABCD    ");
			m.put(o2.getIv(), o2);
   		 	Data o3 = m.get(258);
			assertEquals("iv", 258, o3.getIv());
			assertEquals("xv", 123.789, o3.getXv(), 0.0005);
			assertEquals("sv", "ABCD    ", o3.getSv());
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
    }
}
